Package: sand
Version: 2.0.0
Title: Statistical Analysis of Network Data with R, 2nd Edition
Authors@R: c(person("Eric", "Kolaczyk", role = c("aut", "cre"), email="eric.kolaczyk@gmail.com"),
    person("Gábor", "Csárdi", role = "aut"),
    person("Carolyn", "Kolaczyk", role = "ctb"))
Author: Eric Kolaczyk [aut, cre],
  Gábor Csárdi [aut],
  Carolyn Kolaczyk [ctb]
Maintainer: Eric Kolaczyk <eric.kolaczyk@gmail.com>
Depends: R (>= 3.5.0), igraph, igraphdata
Imports: utils
Suggests: GO.db, GOstats, ROCR, ape, blockmodels, car, eigenmodel,
        ergm, fdrtool, ggplot2, huge, kernlab, lattice, network,
        networkDynamic, networkTomography, ngspatial, org.Sc.sgd.db,
        sna, vioplot
Description: Data sets and code blocks for the book 'Statistical Analysis of 
  Network Data with R, 2nd Edition'.
License: GPL-3
URL: https://github.com/kolaczyk/sand
BugReports: https://github.com/kolaczyk/sand/issues
LazyData: true
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2020-07-01 14:31:46 UTC; erick
Repository: CRAN
Date/Publication: 2020-07-02 07:20:06 UTC
Built: R 4.6.0; ; 2026-04-26 04:46:24 UTC; unix
