
export(install_sand_packages, C2, C3, C4, C5, C6, C7, C8, C9, C10, C11, N, P)

## Import all packages listed as Imports or Depends
import(igraph, igraphdata)

## Imports from base packages
importFrom(utils, capture.output, packageDescription, sessionInfo)

S3method("print", "sand_chapter")
S3method("print", "sand_next")
S3method("print", "sand_print")
